@@ -51,11 +51,11 @@ static char *Splash=
 </td>
 
 <td BGCOLOR=\"#9090F0\">
-<h1><b>Dillo 0.6.5</b></h1>
+<h1><b>Dillo 0.6.6</b></h1>
 
 <hr>
  <h1>Dillo project<br>
-     <h3>Version 0.6.5<BR> <EM>(this is alpha code)</EM> </h3></h1>
+     <h3>Version 0.6.6<BR> <EM>(this is alpha code)</EM> </h3></h1>
 <h4> License: </h4>
 <p>
     This program is free software; you can redistribute it and/or modify
@@ -71,28 +71,8 @@ static char *Splash=
 <hr>
 <h4> Release overview: </h4>
 <BLOCKQUOTE>
- This long awaited release comes with a large list of fixes and improvements.
-Maybe the most notorious is cookies support! Just read
- <code>doc/Cookies.txt</code> to learn about it.
-</BLOCKQUOTE><BLOCKQUOTE>
- Here's a <em>brief</em> from the full ChangeLog:
- <ul>
-  <li>Improved *BSD detection/configuration code.
-  <li>Fixed some leaks and several segfault sources.
-  <li>Reimplemented the cache index with a hash.
-  <li>Allowed enterpress to submit single text-entry forms.
-  <li>Added support for missing Content-Type in HTTP headers (most
-      notable for ebay).
-  <li>Added support for rendering adjacent &lt;BR&gt;, Tabs in &lt;PRE&gt;,
-      and linebreak handling.
-  <li>Bound the space key to PgDn, and 'b' | 'B' to PgUp.
-  <li>Added the following options to dillorc:
-   <ul>
-    <li>Fontname selection
-    <li>Force submit on enterpress
-    <li>Generate missing submit buttons
-   </ul>
- <ul>
+ This is a pure bug-fix release. The main fixes were on cookies,
+the others are detailed in the ChangeLog.
 </BLOCKQUOTE><BLOCKQUOTE>
  Remember that dillo project uses a release model where every new
 browser shall be better than the former.
