@@ -237,7 +237,9 @@ static int Http_connect_socket(ChainLink *Info)
 {
    gint status;
    struct sockaddr_in name;
-   SocketData_t *S = a_Klist_get_data(ValidSocks, (gint)Info->LocalKey);
+   SocketData_t *S;
+
+   S = a_Klist_get_data(ValidSocks, GPOINTER_TO_INT(Info->LocalKey));
 
    /* Some OSes require this...  */
    memset(&name, 0, sizeof(name));
@@ -332,8 +334,10 @@ static gint Http_get(ChainLink *Info, void *Data, void *ExtraData)
 {
    void *link;
    const DilloUrl *Url = Data;
-   SocketData_t *S = a_Klist_get_data(ValidSocks, (gint)Info->LocalKey);
-   gchar hostname[256], *Host = hostname;
+   SocketData_t *S;
+   gchar *hostname;
+
+   S = a_Klist_get_data(ValidSocks, GPOINTER_TO_INT(Info->LocalKey));
 
    /* Reference Info data */
    S->Info = Info;
@@ -342,11 +346,11 @@ static gint Http_get(ChainLink *Info, void *Data, void *ExtraData)
 
    /* Proxy support */
    if (Http_must_use_proxy(Url)) {
-      strncpy(hostname, URL_HOST(HTTP_Proxy), sizeof(hostname));
+      hostname = g_strdup(URL_HOST(HTTP_Proxy));
       S->port = URL_PORT(HTTP_Proxy);
       S->use_proxy = TRUE;
    } else {
-      Host = (gchar *)URL_HOST(Url);
+      hostname = g_strdup(URL_HOST(Url));
       S->port = URL_PORT(Url);
       S->use_proxy = FALSE;
    }
@@ -367,7 +371,9 @@ static gint Http_get(ChainLink *Info, void *Data, void *ExtraData)
    /* Let the DNS engine solve the hostname, and when done,
     * we'll try to connect the socket */
    link = a_Chain_link_new(a_Http_ccc, Info, CCC_FWD, a_Dns_ccc);
-   a_Dns_ccc(OpStart, 1, link, Host, NULL);
+   a_Dns_ccc(OpStart, 1, link, hostname, NULL);
+
+   g_free(hostname);
    return 0;
 }
 
@@ -377,7 +383,7 @@ static gint Http_get(ChainLink *Info, void *Data, void *ExtraData)
 void
  a_Http_ccc(int Op, int Branch, ChainLink *Info, void *Data, void *ExtraData)
 {
-   gint SKey = (gint)Info->LocalKey;
+   gint SKey = GPOINTER_TO_INT(Info->LocalKey);
    SocketData_t *S = a_Klist_get_data(ValidSocks, SKey);
 
    if ( Branch == 1 ) {
@@ -385,7 +391,7 @@ void
       switch (Op) {
       case OpStart:
          SKey = Http_sock_new();
-         Info->LocalKey = (void *) SKey;
+         Info->LocalKey = GINT_TO_POINTER(SKey);
          if (Http_get(Info, Data, ExtraData) < 0) {
             DEBUG_MSG(2, " HTTP: new abort handler! #2\n");
             S = a_Klist_get_data(ValidSocks, SKey);
