@@ -30,6 +30,7 @@
 #include "dicache.h"
 #include "interface.h"
 #include "nav.h"
+#include "cookies.h"
 
 #define NULLKey 0
 
@@ -59,9 +60,8 @@ typedef struct {
 /*
  *  Local data
  */
-/* A list for cached data */
-static CacheData_t *CacheList = NULL;
-static gint CacheListSize = 0, CacheListMax = 32;
+/* A hash for cached data. Holds pointers to CacheData_t structs */
+static GHashTable *CacheHash = NULL;
 
 /* A list for cache clients.
  * Although implemented as a list, we'll call it ClientQueue  --Jcid */
@@ -75,6 +75,36 @@ static void Cache_process_queue(CacheData_t *entry);
 static void Cache_delayed_process_queue(CacheData_t *entry);
 static void Cache_stop_client(gint Key, gint force);
 
+/*
+ * Determine if two hash entries are equal (used by GHashTable)
+ */
+static gint Cache_hash_entry_equal(gconstpointer v1, gconstpointer v2)
+{
+   return (!a_Url_cmp((DilloUrl *)v1, (DilloUrl *)v2));
+}
+
+/*
+ * Determine the hash value given the key (used by GHashTable)
+ */
+static guint Cache_url_hash(gconstpointer key)
+{
+   const char *p = URL_STR((DilloUrl *)key);
+   guint h = *p;
+
+   if (h)
+      for (p += 1; *p != '\0' && *p != '#'; p++)
+         h = (h << 5) - h + *p;
+
+   return h;
+}
+
+/*
+ * Initialize dicache data
+ */
+void a_Cache_init(void)
+{
+   CacheHash = g_hash_table_new(Cache_url_hash, Cache_hash_entry_equal);
+}
 
 /* Client operations ------------------------------------------------------ */
 
@@ -185,11 +215,11 @@ static void Cache_entry_init(CacheData_t *NewEntry, const DilloUrl *Url)
  */
 static CacheData_t *Cache_entry_add(const DilloUrl *Url)
 {
-   /* Allocate memory */
-   a_List_add(CacheList, CacheListSize, sizeof(*CacheList), CacheListMax);
-   /* Set safe values */
-   Cache_entry_init(&CacheList[CacheListSize], Url);
-   return &CacheList[CacheListSize++];
+   CacheData_t *new_entry = g_new(CacheData_t, 1);
+
+   Cache_entry_init(new_entry, Url);  /* Set safe values */
+   g_hash_table_insert(CacheHash, (gpointer)new_entry->Url, new_entry);
+   return new_entry;
 }
 
 /*
@@ -199,28 +229,7 @@ static CacheData_t *Cache_entry_add(const DilloUrl *Url)
  */
 static CacheData_t *Cache_entry_search(const DilloUrl *Url)
 {
-   gint i;
-
-   for ( i = CacheListSize - 1; i >= 0; --i )
-      if ( !a_Url_cmp(Url, CacheList[i].Url) )
-         return (CacheList + i);
-
-   return NULL;
-}
-
-/*
- * Get the data structure for a cached URL
- * If Url isn't cached, return NULL
- * ===> 'Url' must a be a pointer to 'entry->Url' <===
- */
-static CacheData_t *Cache_entry_fast_search(const DilloUrl *Url)
-{
-   gint i;
-
-   for ( i = CacheListSize - 1; i >= 0; --i )
-      if ( Url == CacheList[i].Url )
-         return (CacheList + i);
-   return NULL;
+   return g_hash_table_lookup(CacheHash, Url);
 }
 
 /*
@@ -233,6 +242,7 @@ static void Cache_entry_free(CacheData_t *entry)
    g_string_free(entry->Header, TRUE);
    a_Url_free((DilloUrl *)entry->Location);
    g_free(entry->Data);
+   g_free(entry);
    /* CCCQuery and CCCAnswer are just references */
 }
 
@@ -252,8 +262,8 @@ static gint Cache_entry_remove_raw(CacheData_t *entry, DilloUrl *url)
    a_Dicache_invalidate_entry(url);
 
    /* remove from cache */
+   g_hash_table_remove(CacheHash, entry->Url);
    Cache_entry_free(entry);
-   a_List_remove(CacheList, entry - CacheList, CacheListSize);
    return 1;
 }
 
@@ -418,21 +428,39 @@ gint a_Cache_open_url(void *web, CA_Callback_t Call, void *CbData)
 
 /*
  * Return the pointer to URL 'Data' in the cache.
- * (We also return its size in 'Size')
+ * (We also return its size in 'Size' and take care of redirections)
  */
 char *a_Cache_url_read(const DilloUrl *Url, gint *Size)
 {
-   CacheData_t *entry = Cache_entry_search(Url);
+   CacheData_t **stack = NULL;
+   gint stackSize = 0;
+   gint stackMax = 32;
+   CacheData_t *entry = NULL;
+   gint i;
 
-   if ( !entry ) {
-      *Size = 0;
-      return NULL;
-   }
-   if ( entry->Flags & CA_Redirect && entry->Location )
-      return a_Cache_url_read(entry->Location, Size);
+   do {
+      entry = Cache_entry_search(Url);
+      if (entry && entry->Flags & CA_Redirect && entry->Location) {
+         a_List_add (stack, stackSize, stackMax);
+         stack[stackSize++] = entry;
+         Url = entry->Location;
+      } else
+         break;
 
-   *Size = entry->ValidSize;
-   return (char *) entry->Data;
+      /* Test for a redirection loop */
+      for (i = 0; i < stackSize - 1; i++) {
+         if (stack[i] == entry) {
+            g_warning ("Redirect loop for URL: >%s<\n", URL_STR_(Url));
+            for (i = 0; i < stackSize; i++)
+               stack[i]->Flags |= CA_RedirectLoop;
+            entry = NULL;
+         }
+      }
+   } while (entry);
+
+   g_free (stack);
+   *Size = (entry) ? entry->ValidSize : 0;
+   return (entry) ? (char *) entry->Data : NULL;
 }
 
 /*
@@ -470,6 +498,88 @@ static char *Cache_parse_field(const char *header, const char *fieldname)
    return NULL;
 }
 
+/*
+ * Extract multiple fields from the header.
+ */
+GList *Cache_parse_multiple_fields(const char *header, const char *fieldname)
+{
+   gint i, j;
+   GList *fields = NULL;
+   char *field;
+
+   for (i = 0; header[i]; i++) {
+      /* Search fieldname */
+      for (j = 0; fieldname[j]; j++)
+         if (tolower(fieldname[j]) != tolower(header[i + j]))
+            break;
+      if (fieldname[j]) {
+         /* skip to next line */
+         for (i += j; header[i] != '\n'; i++);
+         continue;
+      }
+
+      i += j;
+      for ( ; header[i] == ' '; i++);
+      if (header[i] == ':' ) {
+         /* Field found! */
+         while (header[++i] == ' ');
+         for (j = 0; header[i + j] != '\n'; j++);
+         field = g_strndup(header + i, j);
+         fields = g_list_append(fields, field);
+      }
+   }
+   return fields;
+}
+
+/*
+ * Detects 'Content-Type' when the server does not supply one.
+ * It uses the magic(5) logic from file(1). Currently, it
+ * only checks the few mime types that Dillo supports.
+ *
+ * 'Data' is a pointer to the first bytes of the raw data.
+ */
+static gchar *Cache_get_type_from_data(void *Data, size_t Size)
+{
+   static gchar *Types[] = {
+      "application/octet-stream",
+      "text/html", "text/plain",
+      "image/gif", "image/png", "image/jpeg",
+   };
+   gint Type = 0;
+   gchar *p = Data;
+   int i;
+
+   /* HTML try */
+   for (i = 0; i < Size && isspace(p[i]); ++i);
+   if ((Size - i >= 5 && !g_strncasecmp(p+i, "<html", 5)) ||
+       (Size - i >= 5 && !g_strncasecmp(p+i, "<head", 5)) ||
+       (Size - i >= 6 && !g_strncasecmp(p+i, "<title", 6)) ||
+       (Size - i >= 14 && !g_strncasecmp(p+i, "<!doctype html", 14))) {
+      Type = 1;
+
+   /* Images */
+   } else if (Size >= 4 && !g_strncasecmp(p, "GIF8", 4)) {
+      Type = 3;
+   } else if (Size >= 4 && !g_strncasecmp(p, "\x89PNG", 4)) {
+      Type = 4;
+   } else if (Size >= 2 && !g_strncasecmp(p, "\xff\xd8", 2)) {
+      /* JPEG has the first 2 bytes set to 0xffd8 in BigEndian - looking
+       * at the character representation should be machine independent. */
+      Type = 5;
+
+   /* Text */
+   } else {
+      /* We'll assume ASCII if chars are below 128 (after all, this
+       * is a last resort when the server doesn't send Content-Type) */
+      for (i = 0; i < Size && i < 256; i++)
+         if ((guchar) p[i] > 127)
+            break;
+      Type = (i < 12 || (guchar) p[i] > 127) ? 0 : 2;
+   }
+
+   return g_strdup(Types[Type]);
+}
+
 /*
  * Scan, allocate, and set things according to header info.
  * (This function needs the whole header to work)
@@ -478,11 +588,7 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
 {
    gchar *header = entry->Header->str;
    gchar *Length, *Type, *location_str;
-
-   /* Get Content-Type */
-   Type = Cache_parse_field(header, "Content-Type");
-// entry->Type = Type ? Type : g_strdup("text/html"); Hack for ebay  --Jcid
-   entry->Type = Type ? Type : g_strdup("application/octet-stream");
+   GList *Cookies;
 
    if ( header[9] == '3' && header[10] == '0' ) {
       /* 30x: URL redirection */
@@ -491,7 +597,7 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
          /* 301 Moved Permanently */
          entry->Flags |= CA_ForceRedirect;
       location_str = Cache_parse_field(header, "Location");
-      entry->Location = a_Url_new(location_str, URL_STR(entry->Url), 0, 0);
+      entry->Location = a_Url_new(location_str, URL_STR_(entry->Url), 0, 0);
       g_free(location_str);
 
    } else if ( strncmp(header + 9, "404", 3) == 0 ) {
@@ -507,6 +613,18 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
          entry->TotalSize = 0;
    }
 
+#ifndef DISABLE_COOKIES
+   /* BUG: If a server feels like mixing Set-Cookie2 and Set-Cookie
+    * responses which aren't identical, then we have a problem. I don't
+    * know if that is a real issue though. */
+   if ( (Cookies = Cache_parse_multiple_fields(header, "Set-Cookie2")) ||
+        (Cookies = Cache_parse_multiple_fields(header, "Set-Cookie")) ) {
+      a_Cookies_set(Cookies, entry->Url);
+      g_list_foreach(Cookies, (GFunc)g_free, NULL);
+      g_list_free(Cookies);
+   }
+#endif /* !DISABLE_COOKIES */
+
    if ( entry->TotalSize > 0 && entry->TotalSize >= entry->ValidSize ) {
       entry->Data = g_malloc(entry->TotalSize);
       memcpy(entry->Data, (char*)io->IOVec.iov_base+HdrLen, io->Status-HdrLen);
@@ -528,6 +646,13 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
       io->IOVec.iov_len  = entry->BuffSize;
       io->Flags &= ~IOFlag_FreeIOVec;
    }
+
+   /* Get Content-Type */
+   if ( (Type = Cache_parse_field(header, "Content-Type")) == NULL ) {
+     DEBUG_HTTP_MSG("Server didn't send Content-Type in header.\n");
+     Type = Cache_get_type_from_data(entry->Data, entry->ValidSize);
+   }
+   entry->Type = Type;
 }
 
 /*
@@ -572,7 +697,7 @@ static void Cache_process_io(int Op, void *VPtr)
    gint Status, len;
    IOData_t *io = VPtr;
    const DilloUrl *Url = io->ExtData;
-   CacheData_t *entry = Cache_entry_fast_search(Url);
+   CacheData_t *entry = Cache_entry_search(Url);
 
    /* Assert a valid entry (not aborted) */
    if ( !entry )
@@ -584,7 +709,7 @@ static void Cache_process_io(int Op, void *VPtr)
    if ( Op == IOClose ) {
       if (entry->Flags & CA_GotLength && entry->TotalSize != entry->ValidSize){
          DEBUG_HTTP_MSG("Content-Length does NOT match message body,\n"
-                        " at: %s\n", URL_STR(entry->Url));
+                        " at: %s\n", URL_STR_(entry->Url));
       }
       entry->Flags |= CA_GotData;
       entry->Flags &= ~CA_Stopped;          /* it may catch up! */
@@ -644,18 +769,24 @@ static gint Cache_redirect(CacheData_t *entry, gint Flags, BrowserWindow *bw)
 {
    DilloUrl *NewUrl;
 
+   /* if there's a redirect loop, stop now */
+   if (entry->Flags & CA_RedirectLoop) {
+     a_Interface_msg(bw, "ERROR: redirect loop for: %s", URL_STR_(entry->Url));
+     return 0;
+   }
+
    if ( ((entry->Flags & CA_Redirect) && entry->Location) &&
         ((entry->Flags & CA_ForceRedirect) || !entry->ValidSize ||
          entry->ValidSize < 1024 ) ) {
 
       DEBUG_MSG(3, ">>>Redirect from: %s\n to %s\n",
-              URL_STR(entry->Url), URL_STR(entry->Location));
+              URL_STR_(entry->Url), URL_STR_(entry->Location));
       DEBUG_MSG(3,"%s", entry->Header->str);
 
       if ( Flags & WEB_RootUrl ) {
          /* Redirection of the main page */
          a_Nav_remove_top_url(bw);
-         NewUrl = a_Url_new(URL_STR(entry->Location),URL_STR(entry->Url),0,0);
+         NewUrl =a_Url_new(URL_STR_(entry->Location),URL_STR_(entry->Url),0,0);
          a_Nav_push(bw, NewUrl);
          a_Url_free(NewUrl);
       } else {
@@ -823,7 +954,7 @@ static void Cache_stop_client(gint Key, gint force)
    if (force &&
        !g_slist_find_custom(ClientQueue, url, Cache_client_url_cmp)) {
       /* it was the last client of this entry */
-      if ((entry = Cache_entry_fast_search(url))) {
+      if ((entry = Cache_entry_search(url))) {
          if (entry->CCCQuery) {
             a_Cache_ccc(OpAbort, -1, entry->CCCQuery, NULL, NULL);
          } else if (entry->CCCAnswer) {
@@ -862,7 +993,7 @@ void
          /* unlink HTTP_Info */
          a_Chain_del_link(Info, CCC_BCK);
          /* 'entry->CCCQuery' and 'Info' point to the same place! */
-         if ((entry = Cache_entry_fast_search(Info->LocalKey)) != NULL)
+         if ((entry = Cache_entry_search(Info->LocalKey)) != NULL)
             entry->CCCQuery = NULL;
          g_free(Info);
          DEBUG_MSG(3, "Cache CCC (OpEnd) [0]\n");
@@ -888,7 +1019,7 @@ void
       switch (Op) {
       case OpStart:
          Info->LocalKey = ExtraData;
-         if ((entry = Cache_entry_fast_search(Info->LocalKey))) {
+         if ((entry = Cache_entry_search(Info->LocalKey))) {
             entry->CCCAnswer = Info;
          } else {
             /* The cache-entry was removed */
@@ -898,7 +1029,7 @@ void
          break;
       case OpSend:
          /* Send data */
-         if ((entry = Cache_entry_fast_search(Info->LocalKey))) {
+         if ((entry = Cache_entry_search(Info->LocalKey))) {
             Cache_process_io(IORead, Data);
          } else {
             a_Chain_bcb(OpAbort, -1, Info, NULL, NULL);
@@ -925,8 +1056,8 @@ void
       switch (Op) {
       case OpStop:
          /* We'll let it fill the entry by now... */
-         // a_Chain_bcb(OpStop, -1, Info, NULL, NULL);
-         // g_free(Info);
+         /* a_Chain_bcb(OpStop, -1, Info, NULL, NULL);
+            g_free(Info); */
          break;
       case OpAbort:
          DEBUG_MSG(2, "Cache: OpAbort [-1]\n");
@@ -938,6 +1069,12 @@ void
    }
 }
 
+static gboolean
+Cache_remove_hash_entry(gpointer key, gpointer value, gpointer user_data)
+{
+   Cache_entry_free((CacheData_t *)value);
+   return TRUE;
+}
 
 
 /*
@@ -945,7 +1082,6 @@ void
  */
 void a_Cache_freeall(void)
 {
-   CacheData_t *entry;
    CacheClient_t *Client;
 
    /* free the client queue */
@@ -953,10 +1089,9 @@ void a_Cache_freeall(void)
       Cache_client_dequeue(Client, NULLKey);
 
    /* free the main cache */
-   while (CacheListSize > 0) {
-      entry = &CacheList[0];
-      Cache_entry_free(entry);
-      a_List_remove(CacheList, 0, CacheListSize);
-   }
-   g_free(CacheList);
+   /* Remove every cache entry */
+   g_hash_table_foreach_remove(CacheHash, (GHRFunc)Cache_remove_hash_entry,
+                               NULL);
+   /* Remove the cache hash */
+   g_hash_table_destroy(CacheHash);
 }
