@@ -19,6 +19,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/file.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -71,8 +72,12 @@ static int num_ccontrol = 0;
 static int num_ccontrol_max = 1;
 static CookieControlAction default_action = COOKIE_DENY;
 
+static gboolean disabled;
+static FILE *file_stream;
+
 static FILE *Cookies_fopen(const char *file, gchar *init_str);
 static CookieControlAction Cookies_control_check(const DilloUrl *url);
+static CookieControlAction Cookies_control_check_domain(const char *domain);
 static void Cookie_control_init();
 static void Cookies_parse_ports(CookieData_t *cookie, const char *port_str);
 static char *Cookies_build_ports_str(CookieData_t *cookie);
@@ -87,7 +92,7 @@ static FILE *Cookies_fopen(const char *filename, gchar *init_str)
    FILE *F_in;
    int fd;
 
-   if ((F_in = fopen(filename, "r")) == NULL) {
+   if ((F_in = fopen(filename, "r+")) == NULL) {
       /* Create the file */
       fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
       if (fd != -1) {
@@ -98,7 +103,7 @@ static FILE *Cookies_fopen(const char *filename, gchar *init_str)
          DEBUG_MSG(10, "Cookies: Created file: %s\n", filename);
          F_in = Cookies_fopen(filename, NULL);
       } else {
-         g_error("Cookies: Could not create file: %s!\n", filename);
+         DEBUG_MSG(10, "Cookies: Could not create file: %s!\n", filename);
       }
    }
 
@@ -123,23 +128,37 @@ void a_Cookies_init()
 {
    CookieData_t *cookie;
    GList *domain_cookies;
-   FILE *F_in;
-   char *file;
+   char *filename;
    char line[LINE_MAXLEN];
 
+   Cookie_control_init();
+
+   /* Get a stream for the cookies file */
+   filename = a_Misc_prepend_user_home(".dillo/cookies");
+   file_stream = Cookies_fopen(filename, NULL);
+   g_free(filename);
+
+   if (!file_stream) {
+      /* Can't open the file, so we disable cookies */
+      disabled = TRUE;
+      return;
+   }
+
+   /* Try to get a lock from the file descriptor */
+   disabled = (lockf(fileno(file_stream), F_TLOCK, 0) == -1);
+   if (disabled) {
+      DEBUG_MSG(10, "The cookies file has a file lock:\n"
+                    "  disabling cookies in this dillo!\n");
+      fclose(file_stream);
+      return;
+   }
+
    cookies = g_hash_table_new(g_str_hash, g_str_equal);
 
-   /* Get a file pointer */
-   file = a_Misc_prepend_user_home(".dillo/cookies");
-   F_in = Cookies_fopen(file, NULL);
-   g_free(file);
-
-   g_hash_table_freeze(cookies);
-
    /* Get all lines in the file */
-   while (!feof(F_in)) {
+   while (!feof(file_stream)) {
       line[0] = '\0';
-      fgets(line, LINE_MAXLEN, F_in);
+      fgets(line, LINE_MAXLEN, file_stream);
 
       /* Remove leading and trailing whitespaces */
       g_strstrip(line);
@@ -159,6 +178,7 @@ void a_Cookies_init()
           * pieces[8] Comment
           * pieces[9] Comment url
           */
+         CookieControlAction action;
          char *piece;
          char *line_marker = line;
 
@@ -182,10 +202,19 @@ void a_Cookies_init()
          cookie->comment = g_strdup(d_strsep(&line_marker, "\t"));
          cookie->comment_url = g_strdup(d_strsep(&line_marker, "\t"));
 
+         action = Cookies_control_check_domain(cookie->domain);
+         if (action == COOKIE_DENY) {
+            Cookies_free_cookie(cookie);
+            continue;
+         } else if (action == COOKIE_ACCEPT_SESSION) {
+            cookie->session_only = TRUE;
+         }
+
          /* Save cookie in memory */
          if (cookie->expires_at > time(NULL)) {
             domain_cookies = (GList *) g_hash_table_lookup(cookies,
                                                            cookie->domain);
+
             if (domain_cookies == NULL) {
                domain_cookies = g_list_append(domain_cookies, cookie);
                g_hash_table_insert(cookies, g_strdup(cookie->domain),
@@ -199,11 +228,6 @@ void a_Cookies_init()
          }
       }
    }
-   fclose(F_in);
-
-   g_hash_table_thaw(cookies);
-
-   Cookie_control_init();
 }
 
 /*
@@ -215,7 +239,6 @@ static gboolean Cookies_freeall_cb(gpointer key,
 {
    CookieData_t *cookie;
    GList *domain_cookies;
-   FILE *F_out = (FILE *) data;
    char *ports_str;
 
    for (domain_cookies = g_list_first((GList *) value);
@@ -224,9 +247,9 @@ static gboolean Cookies_freeall_cb(gpointer key,
       cookie = (CookieData_t *) domain_cookies->data;
 
       if (cookie != NULL) {
-         if (!cookie->session_only && F_out) {
+         if (!cookie->session_only) {
             ports_str = Cookies_build_ports_str(cookie);
-            fprintf(F_out, "%d\t%s\t%s\t%s\t%d\t%ld\t%s\t%s\t%s\t%s\n",
+            fprintf(file_stream, "%d\t%s\t%s\t%s\t%d\t%ld\t%s\t%s\t%s\t%s\n",
                     cookie->version,
                     cookie->domain,
                     ports_str,
@@ -254,21 +277,19 @@ static gboolean Cookies_freeall_cb(gpointer key,
  */
 void a_Cookies_freeall()
 {
-   FILE *F_out;
-   char *file;
+   int fd;
 
-   /* Die if the cookie file isn't writeable. */
-   file = a_Misc_prepend_user_home(".dillo/cookies");
-   if ((F_out = fopen(file, "w")) == NULL)
-      g_error("Cookies: Couldn't open %s for writing!\n", file);
-   g_free(file);
+   if (disabled)
+      return;
 
-   g_hash_table_freeze(cookies);
-   g_hash_table_foreach_remove(cookies, Cookies_freeall_cb, (gpointer)F_out);
-   g_hash_table_thaw(cookies);
+   rewind(file_stream);
+   fd = fileno(file_stream);
+   ftruncate(fd, 0);
 
-   if (F_out)
-      fclose(F_out);
+   g_hash_table_foreach_remove(cookies, Cookies_freeall_cb, NULL);
+
+   lockf(fd, F_ULOCK, 0);
+   fclose(file_stream);
 }
 
 /*
@@ -291,7 +312,7 @@ static char *Cookies_try_header(char **string, char *header, gboolean compat)
    marker = strchr(value_start, '=');
    if (marker == NULL) {
       *string = NULL;
-      return g_strdup(""); /* Some attributes doesn't use '=' */
+      return g_strdup(""); /* Some attributes don't use '=' */
    }
    marker++;
    g_strstrip(marker);
@@ -344,47 +365,54 @@ static int Cookies_get_month(const char *month_name)
 }
 
 /*
- * Returns a local timestamp from an GMT date string
+ * Return a local timestamp from a GMT date string
+ * Accept: RFC-1123 | RFC-850 | ANSI asctime
+ * (return 0 on malformed date string syntax)
  */
 static time_t Cookies_create_timestamp(const char *expires)
 {
+   gchar *E_msg = "Cookies: Expire date is malformed! Ignoring cookie...\n";
    time_t ret;
    int day, month, year, hour, minutes, seconds;
-   char *cp;
+   gchar *cp;
 
-   cp = (char *)expires;
+   if (!(cp = strchr(expires, ',')) && strlen(expires) == 24) {
+      /* Looks like ANSI asctime format... */
+      cp = (gchar *)expires;
+      day = strtol(cp + 8, NULL, 10);       /* day */
+      month = Cookies_get_month(cp + 4);    /* month */
+      year = strtol(cp + 20, NULL, 10);     /* year */
+      hour = strtol(cp + 11, NULL, 10);     /* hour */
+      minutes = strtol(cp + 14, NULL, 10);  /* minutes */
+      seconds = strtol(cp + 17, NULL, 10);  /* seconds */
 
-   /* Skip past the weekday */
-   while (*(cp++) != ',')
-      ;
-   cp++;
+   } else if (cp && ((cp - expires == 3 && strlen(cp) == 26) ||
+                     (cp - expires > 5  && strlen(cp) == 24))) {
+      /* RFC-1123 | RFC-850 format */
+      day = strtol(cp + 2, NULL, 10);
+      month = Cookies_get_month(cp + 5);
+      year = strtol(cp + 9, &cp, 10);
+      /* todo: tricky, because two digits for year IS ambiguous! */
+      year += (year < 70) ? 2000 : ((year < 100) ? 1900 : 0);
+      hour = strtol(cp + 1, NULL, 10);
+      minutes = strtol(cp + 4, NULL, 10);
+      seconds = strtol(cp + 7, NULL, 10);
 
-   /* Get day of month */
-   day = strtol(cp, &cp, 10);
-   cp++;
+   } else {
+      DEBUG_MSG(5, E_msg);
+      return (time_t) 0;
+   }
 
-   /* Get the month */
-   month = Cookies_get_month(cp);
-   cp += 4;
-
-   /* Get the year */
-   year = strtol(cp, &cp, 10);
-   if (year < 70)
-      year += 2000;
-   cp++;
-
-   /* Hours */
-   hour = strtol(cp, &cp, 10);
-   cp++;
-
-   /* Minutes */
-   minutes = strtol(cp, &cp, 10);
-   cp++;
-
-   /* Seconds */
-   seconds = strtol(cp, &cp, 10);
-
-   /* Stolen from Lynx... (http://lynx.browser.org) */
+   /* Error checks  --this may be overkill */
+   if (!(day > 0 && day < 32 && month > 0 && month && year > 1970 &&
+         hour >= 0 && hour < 24 && minutes >= 0 && minutes < 60 &&
+         seconds >= 0 && seconds < 60)) {
+      DEBUG_MSG(5, E_msg);
+      return (time_t) 0;
+   }
+   
+   /* Calculate local timestamp.
+    * [stolen from Lynx... (http://lynx.browser.org)] */
    month -= 3;
    if (month < 0) {
       month += 12;
@@ -686,13 +714,38 @@ static gboolean Cookies_validate_domain(CookieData_t *cookie,
    return TRUE;
 }
 
+/*
+ * Strip of the filename from a full path
+ */
+static char *Cookies_strip_path(const char *path)
+{
+   char *ret;
+   int len;
+
+   if (path) {
+      len = strlen(path);
+
+      while(len && path[len] != '/')
+         len--;
+      ret = g_strndup(path, len + 1);
+   } else {
+      ret = g_strdup("/");
+   }
+
+   return ret;
+}
+
 /*
  * Set the value corresponding to the cookie string
  */
 void a_Cookies_set(GList *cookie_strings, const DilloUrl *set_url)
 {
-   CookieControlAction action = Cookies_control_check(set_url);
+   CookieControlAction action;
 
+   if (disabled)
+      return;
+
+   action = Cookies_control_check(set_url);
    if (action == COOKIE_DENY) {
       DEBUG_MSG(5, "Cookies: denied SET for %s\n", URL_HOST_(set_url));
       return;
@@ -724,8 +777,7 @@ void a_Cookies_set(GList *cookie_strings, const DilloUrl *set_url)
 
       if (Cookies_validate_domain(cookie, set_url)) {
          if (cookie->path == NULL) {
-            cookie->path = URL_PATH_(set_url) ? 
-                              g_dirname(URL_PATH_(set_url)) : g_strdup("/");
+            cookie->path = Cookies_strip_path(URL_PATH_(set_url));
          }
          domain_cookies = (GList *)g_hash_table_lookup(cookies,
                                                        cookie->domain);
@@ -755,7 +807,9 @@ void a_Cookies_set(GList *cookie_strings, const DilloUrl *set_url)
             }
          }
          /* If this is non-expiring, add it */
-         if (cookie->session_only || cookie->expires_at > time(NULL)) {
+         if (g_list_length(domain_cookies) < 20 &&
+             (cookie->session_only || cookie->expires_at > time(NULL))) {
+
             DEBUG_MSG(5, "Adding cookie '%s' for %s\n",
                       cookie->name, cookie->domain);
             domain_cookies = g_list_append(domain_cookies, cookie);
@@ -799,14 +853,16 @@ char *a_Cookies_get(const DilloUrl *request_url)
    gboolean cookies_removed;
    GString *cookie_gstring;
 
+   if (disabled)
+      return g_strdup("");
+
    action = Cookies_control_check(request_url);
    if (action == COOKIE_DENY) {
       DEBUG_MSG(5, "Cookies: denied GET for %s\n", URL_HOST_(request_url));
       return g_strdup("");
    }
 
-   path = (URL_PATH_(request_url)) ? g_dirname(URL_PATH_(request_url)) :
-      g_strdup("/");
+   path = Cookies_strip_path(URL_PATH_(request_url));
 
    /* Check if the protocol is secure or not */
    is_ssl = (!g_strcasecmp(URL_SCHEME(request_url), "https"));
@@ -937,6 +993,9 @@ static void Cookie_control_init()
    stream = Cookies_fopen(filename, "DEFAULT DENY\n");
    g_free(filename);
 
+   if (!stream)
+      return;
+
    /* Get all lines in the file */
    while (!feof(stream)) {
       line[0] = '\0';
@@ -993,23 +1052,23 @@ static void Cookie_control_init()
 }
 
 /*
- * Match a url with the rules
+ * Check the rules for an appropriate action for this domain
  */
-static CookieControlAction Cookies_control_check(const DilloUrl *url)
+static CookieControlAction Cookies_control_check_domain(const char *domain)
 {
    int i, diff;
 
    for (i = 0; i < num_ccontrol; i++) {
       if (ccontrol[i].domain[0] == '.') {
-         diff = strlen(URL_HOST(url)) - strlen(ccontrol[i].domain);
-         if (diff > 0) {
-            if (g_strcasecmp((URL_HOST(url) + diff), ccontrol[i].domain) != 0)
+         diff = strlen(domain) - strlen(ccontrol[i].domain);
+         if (diff >= 0) {
+            if (g_strcasecmp(domain + diff, ccontrol[i].domain) != 0)
                continue;
          } else {
             continue;
          }
       } else {
-         if (g_strcasecmp(URL_HOST(url), ccontrol[i].domain) != 0)
+         if (g_strcasecmp(domain, ccontrol[i].domain) != 0)
             continue;
       }
 
@@ -1017,7 +1076,15 @@ static CookieControlAction Cookies_control_check(const DilloUrl *url)
       return( ccontrol[i].action );
    }
 
-   return( default_action );
+   return default_action;
+}
+
+/*
+ * Same as the above except it takes an URL
+ */
+static CookieControlAction Cookies_control_check(const DilloUrl *url)
+{
+   return Cookies_control_check_domain(URL_HOST(url));
 }
 
 #endif /* !DISABLE_COOKIES */
