@@ -4,6 +4,6 @@
 #include "browser.h"
 #include "web.h"
 
-#define DILLO_HOME "http://dillo.sourceforge.net/"
+#define DILLO_HOME "http://dillo.cipsga.org.br/"
 
 #endif /* __DILLO_H__ */
