@@ -271,9 +271,9 @@ static void Dw_gtk_viewport_paint (GtkWidget *widget,
 
          /* Draw top-level Dw widget. */
          parent_area.x =
-            Dw_widget_x_viewport_to_world (viewport->child, area->x);
+            p_Dw_widget_x_viewport_to_world (viewport->child, area->x);
          parent_area.y =
-            Dw_widget_y_viewport_to_world (viewport->child, area->y);
+            p_Dw_widget_y_viewport_to_world (viewport->child, area->y);
          parent_area.width = area->width;
          parent_area.height = area->height;
 
@@ -283,8 +283,8 @@ static void Dw_gtk_viewport_paint (GtkWidget *widget,
          child_area.height = (viewport->child->allocation.ascent +
                               viewport->child->allocation.descent);
 
-         if (Dw_rectangle_intersect (&parent_area, &child_area,
-                                     &intersection)) {
+         if (p_Dw_rectangle_intersect (&parent_area, &child_area,
+                                       &intersection)) {
             intersection.x -= viewport->child->allocation.x;
             intersection.y -= viewport->child->allocation.y;
 
@@ -744,9 +744,10 @@ static void Dw_gtk_viewport_update_anchor_rec (DwWidget *widget)
    viewport = GTK_DW_VIEWPORT (widget->viewport);
 
    if (widget->anchors_table &&
-       (p = g_hash_table_lookup (widget->anchors_table, viewport->anchor)))
+       g_hash_table_lookup_extended (widget->anchors_table, viewport->anchor,
+                                     NULL, &p))
       Dw_gtk_viewport_scroll_to (viewport,
-				 GPOINTER_TO_INT(p) + widget->allocation.y);
+                                 GPOINTER_TO_INT(p) + widget->allocation.y);
    else {
       if (DW_IS_CONTAINER (widget))
          a_Dw_container_forall (DW_CONTAINER (widget),
