@@ -46,8 +46,10 @@ static gint Dw_image_motion_notify      (DwWidget *widget,
                                          gint32 y,
                                          GdkEventMotion *event);
 static gint Dw_image_enter_notify       (DwWidget *widget,
+                                         DwWidget *last_widget,
                                          GdkEventMotion *event);
 static gint Dw_image_leave_notify       (DwWidget *widget,
+                                         DwWidget *next_widget,
                                          GdkEventMotion *event);
 
 static void Dw_image_find_link          (DwImage *image,
@@ -66,7 +68,7 @@ static gint Dw_image_map_list_find_link (DwImageMapList *list,
 #define Dw_image_scaled_y(image, y_src) \
    ( (y_src) * ( ((DwWidget*)(image))->allocation.ascent +  \
                  ((DwWidget*)(image))->allocation.descent - \
-                 Dw_style_box_diff_height ( ((DwWidget*)(image))->style ) ) \
+                 p_Dw_style_box_diff_height ( ((DwWidget*)(image))->style ) ) \
              / ((DwImage*)(image))->height )
 
 
@@ -156,7 +158,7 @@ static void Dw_image_class_init (DwImageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_entered),
-                      Dw_marshal_link_enter,
+                      p_Dw_marshal_link_enter,
                       GTK_TYPE_NONE,
                       3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT);
    image_signals[LINK_PRESSED] =
@@ -164,7 +166,7 @@ static void Dw_image_class_init (DwImageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_pressed),
-                      Dw_marshal_link_button,
+                      p_Dw_marshal_link_button,
                       GTK_TYPE_NONE,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
@@ -173,7 +175,7 @@ static void Dw_image_class_init (DwImageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_released),
-                      Dw_marshal_link_button,
+                      p_Dw_marshal_link_button,
                       GTK_TYPE_NONE,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
@@ -182,7 +184,7 @@ static void Dw_image_class_init (DwImageClass *klass)
                       GTK_RUN_FIRST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_clicked),
-                      Dw_marshal_link_button,
+                      p_Dw_marshal_link_button,
                       GTK_TYPE_NONE,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
@@ -229,9 +231,10 @@ static void Dw_image_size_request (DwWidget *widget,
    DwImage *image;
 
    image = DW_IMAGE (widget);
-   requisition->width = image->width + Dw_style_box_diff_width (widget->style);
+   requisition->width =
+      image->width + p_Dw_style_box_diff_width (widget->style);
    requisition->ascent =
-      image->height + Dw_style_box_diff_height (widget->style);
+      image->height + p_Dw_style_box_diff_height (widget->style);
    requisition->descent = 0;
 }
 
@@ -276,19 +279,19 @@ static void Dw_image_draw (DwWidget *widget,
    //g_print(">Area x=%d y=%d w=%d h=%d\n", area->x, area->y,
    //        area->width, area->height);
 
-   Dw_widget_draw_widget_box (widget, area);
+   p_Dw_widget_draw_widget_box (widget, area);
 
    if (image->buffer) {
-      content.x = Dw_style_box_offset_x (widget->style);
-      content.y = Dw_style_box_offset_y (widget->style);
+      content.x = p_Dw_style_box_offset_x (widget->style);
+      content.y = p_Dw_style_box_offset_y (widget->style);
       content.width =
-         widget->allocation.width - Dw_style_box_diff_width (widget->style);
+         widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
       content.height = widget->allocation.ascent + widget->allocation.descent -
-         Dw_style_box_diff_height (widget->style);
+         p_Dw_style_box_diff_height (widget->style);
 
-      if (Dw_rectangle_intersect (area, &content, &intersection)) {
-         vx = Dw_widget_x_world_to_viewport (widget, widget->allocation.x);
-         vy = Dw_widget_y_world_to_viewport (widget, widget->allocation.y);
+      if (p_Dw_rectangle_intersect (area, &content, &intersection)) {
+         vx = p_Dw_widget_x_world_to_viewport (widget, widget->allocation.x);
+         vy = p_Dw_widget_y_world_to_viewport (widget, widget->allocation.y);
          gc = widget->style->color->gc;
 
          if (image->scaled_buffer)
@@ -297,9 +300,9 @@ static void Dw_image_draw (DwWidget *widget,
             buffer = image->buffer;
 
          bstart = buffer +
-            3 * ( intersection.x - Dw_style_box_offset_x (widget->style)+
+            3 * ( intersection.x - p_Dw_style_box_offset_x (widget->style)+
                   content.width * (intersection.y -
-                                   Dw_style_box_offset_y (widget->style)) );
+                                   p_Dw_style_box_offset_y (widget->style)) );
 
          gdk_draw_rgb_image(
             DW_WIDGET_WINDOW (widget), gc,
@@ -315,6 +318,7 @@ static void Dw_image_draw (DwWidget *widget,
  * Standard Dw function.
  */
 static gint Dw_image_enter_notify (DwWidget *widget,
+                                   DwWidget *last_widget,
                                    GdkEventMotion *event)
 {
    DwImage *image = DW_IMAGE (widget);
@@ -329,6 +333,7 @@ static gint Dw_image_enter_notify (DwWidget *widget,
  * Standard Dw function.
  */
 static gint Dw_image_leave_notify (DwWidget *widget,
+                                   DwWidget *next_widget,
                                    GdkEventMotion *event)
 {
    DwImage *image = DW_IMAGE (widget);
@@ -446,8 +451,8 @@ static void Dw_image_find_link (DwImage *image,
    if (image->ismap &&
        (style = DW_WIDGET(image)->style) && style->link != -1) {
       *link = style->link;
-      *link_x = x - Dw_style_box_offset_x (widget->style);
-      *link_y = y - Dw_style_box_offset_y (widget->style);
+      *link_x = x - p_Dw_style_box_offset_x (widget->style);
+      *link_y = y - p_Dw_style_box_offset_y (widget->style);
       return;
    }
 
@@ -465,7 +470,7 @@ void a_Dw_image_size(DwImage *image, gint width, gint height)
    image->width = width;
    image->height = height;
    if ( Resize )
-      Dw_widget_queue_resize(DW_WIDGET (image), 0, TRUE);
+      p_Dw_widget_queue_resize(DW_WIDGET (image), 0, TRUE);
 
    if (image->buffer)
       /* if a_Dw_image_set_buffer has been called before */
@@ -490,13 +495,15 @@ void a_Dw_image_draw_row(DwImage *image,
 
       dy1 = Dw_image_scaled_y (image, y);
       dy2 = Dw_image_scaled_y (image, y + 1);
-      Dw_widget_queue_draw_area(widget,
-                                x, dy1 + Dw_style_box_offset_y (widget->style),
-                                widget->allocation.width, dy2 - dy1);
+      p_Dw_widget_queue_draw_area(widget,
+                                  x,
+                                  dy1 + p_Dw_style_box_offset_y(widget->style),
+                                  widget->allocation.width, dy2 - dy1);
    } else
-      Dw_widget_queue_draw_area (widget,
-                                 x, y + Dw_style_box_offset_y (widget->style),
-                                 widget->allocation.width, 1);
+      p_Dw_widget_queue_draw_area (widget,
+                                   x,
+                                   y + p_Dw_style_box_offset_y (widget->style),
+                                   widget->allocation.width, 1);
 }
 
 /*
@@ -551,9 +558,9 @@ static void Dw_image_scale (DwImage *image)
    }
 
    widget = DW_WIDGET (image);
-   w = widget->allocation.width - Dw_style_box_diff_width (widget->style);
+   w = widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
    h = widget->allocation.ascent + widget->allocation.descent -
-      Dw_style_box_diff_height (widget->style);
+      p_Dw_style_box_diff_height (widget->style);
 
    /* Zero or negative sizes? Ignore. */
    if (w <= 0 || h <= 0)
@@ -580,7 +587,8 @@ static void Dw_image_scale_row (DwImage *image, gint y_src)
 
    widget = DW_WIDGET (image);
    w_src = image->width;
-   w_dest = widget->allocation.width - Dw_style_box_diff_width (widget->style);
+   w_dest =
+      widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
    y_dest1 = Dw_image_scaled_y (image, y_src);
    y_dest2 = Dw_image_scaled_y (image, y_src + 1);
 
@@ -676,8 +684,7 @@ void a_Dw_image_map_list_add_map (DwImageMapList *list,
                                   DilloUrl *url)
 {
    list->num_maps++;
-   a_List_add(list->maps, list->num_maps, sizeof(DwImageMap),
-              list->num_maps_max);
+   a_List_add(list->maps, list->num_maps, list->num_maps_max);
    list->maps[list->num_maps - 1].url = a_Url_dup (url);
    list->maps[list->num_maps - 1].start_shape = list->num_shapes;
 }
@@ -711,8 +718,7 @@ void a_Dw_image_map_list_add_shape (DwImageMapList *list,
 
    if (correct_args) {
       list->num_shapes++;
-      a_List_add(list->shapes, list->num_shapes, sizeof(DwImageMapShape),
-              list->num_shapes_max);
+      a_List_add(list->shapes, list->num_shapes, list->num_shapes_max);
       shape = &list->shapes[list->num_shapes - 1];
       shape->type = type;
       shape->link = link;
@@ -753,7 +759,7 @@ static gint Dw_image_map_list_find_link (DwImageMapList *list,
 
    for (i = 0; i < list->num_maps; i++) {
       if (a_Url_cmp (list->maps[i].url, url) == 0 &&
-          URL_STRCAMP_EQ(URL_FRAGMENT(list->maps[i].url), URL_FRAGMENT(url))) {
+          URL_STRCAMP_EQ(URL_FRAGMENT_(list->maps[i].url),URL_FRAGMENT_(url))){
          /* map found */
          start = list->maps[i].start_shape;
          if (i == list->num_maps - 1)
