@@ -5,15 +5,15 @@ typedef void (*GtkSignal_NONE__INT_INT_INT) (GtkObject * object,
                                              gint arg1, gint arg2,
                                              gint arg3, gpointer user_data);
 
-void Dw_marshal_NONE__INT_INT_INT (GtkObject * object,
-                                   GtkSignalFunc func,
-                                   gpointer func_data, GtkArg * args)
+void p_Dw_marshal_NONE__INT_INT_INT (GtkObject * object,
+                                     GtkSignalFunc Func,
+                                     gpointer FuncData, GtkArg * args)
 {
    GtkSignal_NONE__INT_INT_INT rfunc;
-   rfunc = (GtkSignal_NONE__INT_INT_INT) func;
+   rfunc = (GtkSignal_NONE__INT_INT_INT) Func;
    rfunc (object,
           GTK_VALUE_INT (args[0]), GTK_VALUE_INT (args[1]),
-          GTK_VALUE_INT (args[2]), func_data);
+          GTK_VALUE_INT (args[2]), FuncData);
 }
 
 
@@ -23,14 +23,14 @@ typedef void (*GtkSignal_NONE__INT_INT_INT_POINTER) (GtkObject * object,
                                                      gpointer arg4,
                                                      gpointer user_data);
 
-void Dw_marshal_NONE__INT_INT_INT_POINTER (GtkObject * object,
-                                           GtkSignalFunc func,
-                                           gpointer func_data, GtkArg * args)
+void p_Dw_marshal_NONE__INT_INT_INT_POINTER (GtkObject * object,
+                                             GtkSignalFunc Func,
+                                             gpointer FuncData, GtkArg * args)
 {
    GtkSignal_NONE__INT_INT_INT_POINTER rfunc;
-   rfunc = (GtkSignal_NONE__INT_INT_INT_POINTER) func;
+   rfunc = (GtkSignal_NONE__INT_INT_INT_POINTER) Func;
    rfunc (object,
           GTK_VALUE_INT (args[0]), GTK_VALUE_INT (args[1]),
-          GTK_VALUE_INT (args[2]), GTK_VALUE_POINTER (args[3]), func_data);
+          GTK_VALUE_INT (args[2]), GTK_VALUE_POINTER (args[3]), FuncData);
 }
 
