@@ -657,7 +657,7 @@ DwStyleColor* a_Dw_style_color_new (gint color_val,
 {
    DwStyleColor *color;
 
-   color = g_hash_table_lookup (colors_table, (gconstpointer)color_val);
+   color = g_hash_table_lookup (colors_table, GINT_TO_POINTER(color_val));
    if (color == NULL) {
       color = g_new (DwStyleColor, 1);
       color->ref_count = 0;
@@ -665,7 +665,7 @@ DwStyleColor* a_Dw_style_color_new (gint color_val,
 
       Dw_style_color_create (color_val, window,
                              &color->color, &color->gc, 0);
-      g_hash_table_insert (colors_table, (gpointer)color_val, color);
+      g_hash_table_insert (colors_table, GINT_TO_POINTER(color_val), color);
    }
 
    return color;
@@ -676,7 +676,7 @@ DwStyleColor* a_Dw_style_color_new (gint color_val,
  */
 static void Dw_style_color_remove (DwStyleColor *color)
 {
-   g_hash_table_remove (colors_table, (gpointer)color->color_val);
+   g_hash_table_remove (colors_table, GINT_TO_POINTER(color->color_val));
    gdk_gc_destroy (color->gc);
    g_free (color);
 }
@@ -691,7 +691,7 @@ DwStyleShadedColor* a_Dw_style_shaded_color_new (gint color_val,
 {
    DwStyleShadedColor *color;
 
-   color = g_hash_table_lookup (shaded_colors_table, (gconstpointer)color_val);
+   color = g_hash_table_lookup(shaded_colors_table,GINT_TO_POINTER(color_val));
    if (color == NULL) {
       color = g_new (DwStyleShadedColor, 1);
       color->ref_count = 0;
@@ -703,7 +703,8 @@ DwStyleShadedColor* a_Dw_style_shaded_color_new (gint color_val,
                              &color->color_dark, &color->gc_dark, -1);
       Dw_style_color_create (color_val, window,
                              &color->color_light, &color->gc_light, +1);
-      g_hash_table_insert (shaded_colors_table, (gpointer)color_val, color);
+      g_hash_table_insert (shaded_colors_table,
+                           GINT_TO_POINTER(color_val), color);
    }
 
    return color;
@@ -714,7 +715,7 @@ DwStyleShadedColor* a_Dw_style_shaded_color_new (gint color_val,
  */
 static void Dw_style_shaded_color_remove (DwStyleShadedColor *color)
 {
-   g_hash_table_remove (shaded_colors_table, (gpointer)color->color_val);
+   g_hash_table_remove(shaded_colors_table, GINT_TO_POINTER(color->color_val));
    gdk_gc_destroy (color->gc);
    gdk_gc_destroy (color->gc_dark);
    gdk_gc_destroy (color->gc_light);
