@@ -41,7 +41,6 @@ struct _DilloHtmlLB {
   gint num_links_max;
 
   DwImageMapList maps;
-  gboolean map_open;
 
   gint32 link_color;
   gint32 visited_color;
@@ -83,9 +82,13 @@ typedef enum {
 
 
 typedef enum {
-  DILLO_HTML_IN_HTML,
-  DILLO_HTML_IN_HEAD,
-  DILLO_HTML_IN_BODY
+  IN_HTML        = 1 << 0,
+  IN_HEAD        = 1 << 1,
+  IN_BODY        = 1 << 2,
+  IN_FORM        = 1 << 3,
+  IN_SELECT      = 1 << 4,
+  IN_TEXTAREA    = 1 << 5,
+  IN_MAP         = 1 << 6
 } DilloHtmlProcessingState;
 
 
@@ -186,7 +189,7 @@ struct _DilloHtml {
   gint stack_top;        /* Index to the top of the stack [0 based] */
   gint stack_max;
 
-  DilloHtmlProcessingState InTag; /* where in the document we are */
+  DilloHtmlProcessingState InFlags; /* tracks which tags we are in */
 
   GString *Stash;
   gboolean StashSpace;
@@ -194,8 +197,6 @@ struct _DilloHtml {
   gint pre_column;        /* current column, used in PRE tags with tabs */
   gboolean PreFirstChar;  /* used to skip the first CR or CRLF in PRE tags*/
   gboolean PrevWasCR;     /* Flag to help parsing of "\r\n" in PRE tags */
-  gboolean InForm;
-  gboolean InSelect;
   gboolean InVisitedLink; /* used to 'force_visited_colors' */
 
   GString *attr_data;
