@@ -163,9 +163,9 @@ static gint Interface_sens_idle_func(BrowserWindow *bw)
    gtk_widget_set_sensitive(bw->stop_button, stop_sensitive);
 
    /* Back and Forward buttons */
-   back_sensitive = bw->nav_stack_ptr > 0;
+   back_sensitive = a_Nav_stack_ptr(bw) > 0;
    gtk_widget_set_sensitive(bw->back_button, back_sensitive);
-   forw_sensitive = (bw->nav_stack_ptr < bw->nav_stack_size - 1 &&
+   forw_sensitive = (a_Nav_stack_ptr(bw) < a_Nav_stack_size(bw) - 1 &&
                      !bw->nav_expecting);
    gtk_widget_set_sensitive(bw->forw_button, forw_sensitive);
 
@@ -199,15 +199,13 @@ void a_Interface_add_client(BrowserWindow *bw, gint Key, gint Root)
 
    if ( Root ) {
       nc = bw->NumRootClients;
-      a_List_add(bw->RootClients, nc, sizeof(*bw->RootClients),
-                 bw->MaxRootClients);
+      a_List_add(bw->RootClients, nc, bw->MaxRootClients);
       bw->RootClients[nc] = Key;
       bw->NumRootClients++;
       a_Interface_set_button_sens(bw);
    } else {
       nc = bw->NumImageClients;
-      a_List_add(bw->ImageClients, nc, sizeof(*bw->ImageClients),
-                 bw->MaxImageClients);
+      a_List_add(bw->ImageClients, nc, bw->MaxImageClients);
       bw->ImageClients[nc] = Key;
       bw->NumImageClients++;
       bw->NumImages++;
@@ -241,7 +239,7 @@ void a_Interface_add_url(BrowserWindow *bw, const DilloUrl *Url, gint Flags)
       }
    }
    if ( !found ) {
-      a_List_add(bw->PageUrls, nu, sizeof(*bw->PageUrls), bw->MaxPageUrls);
+      a_List_add(bw->PageUrls, nu, bw->MaxPageUrls);
       bw->PageUrls[nu].Url = a_Url_dup(Url);
       bw->PageUrls[nu].Flags = Flags;
       bw->NumPageUrls++;
@@ -296,7 +294,9 @@ static gboolean Interface_quit(GtkWidget *widget, BrowserWindow *bw)
          break;
       }
 
-   g_free(bw->nav_stack);
+   /* free nav_stack and nav_expect stuff */
+   a_Nav_free(bw);
+
    g_free(bw->RootClients);
    g_free(bw->ImageClients);
 
@@ -575,7 +575,7 @@ BrowserWindow *a_Interface_browser_window_new(gint width, gint height)
    /* We use g_new0() to zero the memory */
    bw = g_new0(BrowserWindow, 1);
 
-   a_List_add(browser_window, num_bw, sizeof(*browser_window), num_bw_max);
+   a_List_add(browser_window, num_bw, num_bw_max);
    browser_window[num_bw++] = bw;
 
    /* initialize nav_stack struct in browser_window struct */
@@ -913,26 +913,37 @@ static void
 
 /*
  * Open an URL specified in the location entry, or in the open URL dialog.
- * The URL is not sent "as is". It's fully parsed by a_Url_new().
+ * The URL is not sent "as is", illegal chars are ripped out,
+ * then it's fully parsed by a_Url_new().
  */
 void a_Interface_entry_open_url(GtkWidget *widget, BrowserWindow *bw)
 {
-   gchar *text;
+   gchar *text, *new_text, *p;
    DilloUrl *url;
    GtkEntry *entry;
 
-   /* widget = { bw->location | bw->open_dialog_entry } */
-   entry = GTK_ENTRY(widget);
+   /* entry = { bw->location | bw->open_dialog_entry } */
+   entry = GTK_ENTRY(widget == bw->location ? widget : bw->open_dialog_entry);
    text = gtk_entry_get_text(entry);
 
    DEBUG_MSG(1, "entry_open_url %s\n", text);
 
    if ( text && *text ) {
-      url = a_Url_new(text, NULL, 0, 0);
+
+      /* This filtering might seem like an overhead, but it aims towards
+       * easier copying&pasting URL-texts into dillo */
+      new_text = g_malloc((strlen(text)+1)*sizeof(gchar));
+      for (p = new_text; *text; text++)
+         if (*text > 0x1F && *text != 0x7F && *text != ' ')
+            *p++ = *text;
+      *p = 0;
+
+      url = a_Url_new(new_text, NULL, 0, 0);
       if ( url ) {
          a_Nav_push(bw, url);
          a_Url_free(url);
       }
+      g_free(new_text);
    }
 
    if (bw->open_dialog_window != NULL)
@@ -1142,7 +1153,7 @@ static void Interface_save_callback(int Op, CacheClient_t *Client)
  */
 static void Interface_file_save_url(GtkWidget *widget, BrowserWindow *bw)
 {
-   const char *url_str, *name;
+   const char *name;
    GtkFileSelection *choosefile;
    GtkEntry *entry_url;
    DilloUrl *url;
@@ -1151,8 +1162,7 @@ static void Interface_file_save_url(GtkWidget *widget, BrowserWindow *bw)
    choosefile = GTK_FILE_SELECTION(bw->save_dialog_window);
    entry_url = GTK_ENTRY(bw->location);
    name = gtk_file_selection_get_filename(choosefile);
-   url_str = gtk_entry_get_text(entry_url);
-   url = a_Url_new(url_str, NULL, 0, 0);
+   url = a_Url_dup(a_History_get_url(NAV_TOP(bw)));
 
    if ( strlen(name) && (out = fopen(name, "w")) != NULL ) {
       DilloWeb *Web = a_Web_new(url);
@@ -1199,7 +1209,7 @@ static char *Interface_make_save_name(const DilloUrl *url)
 {
    gchar *FileName;
 
-   if (URL_PATH(url) && (FileName = strrchr(URL_PATH(url), '/')))
+   if ((FileName = strrchr(URL_PATH(url), '/')))
       return g_strndup(++FileName, MIN(strlen(FileName), 64));
    return g_strdup("");
 }
